/*
  license GPL v3 ...........

  description of this file:
  a page for viewing a user selected RSS feed ;

*/

import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import "./dateutils.js" as DateUtils
import "databasemodule_v2.js" as DB

Page {
     objectName: "rssfeedpage"
    id: pageRoot

    signal articleStatusChanged(int tagId, int articleId, string status)
    signal articleFavouriteChanged(var article, string favourite)

    property string rssModelUrl: ""
    property string rssTitle: ""
    property var rssModel
    property var rssItem: null
    property bool preventIndexChangeHandler: false

    function setNewIndex(rssIndex) {
        rssListview.currentIndex = rssIndex
        rssListview.positionViewAtIndex(rssListview.currentIndex, ListView.Center)
    }

    function setFeed(model, index) {
        /* Setting new model and not-null index will cause two change events instead of one.
         * Settings "preventIndexChangeHandler" to true helps to avoid it.
         */
        if (rssModel != model && index !== 0)
            preventIndexChangeHandler = true
        rssModel = model
        setNewIndex(index)
    }

    function reloadPageContent() {

    }

    function showNextArticle() {
        var index = rssListview.currentIndex + 1
        if (index < rssListview.model.count) {
            rssListview.currentIndex = index
        }
    }

    function showPrevArticle() {
        var index = rssListview.currentIndex - 1
        if (index >= 0) {
            rssListview.currentIndex = index
        }
    }

    tools: ToolbarItems {
        objectName: "pageBackBtn"
        id: toolbar

        ToolbarButton {
            id: readingOptionsButton
            /* QtRoS - Set visibility to true in line below to see options popup.
             * There are no guides on dark and light theme yet, as on font size.
             * So it is just logic for storing settings without any visual effects.
             */
            visible: false
            action: Action {
                text:  i18n.tr("Options")
                iconSource: Qt.resolvedUrl("./icons_tmp/settings.svg")
                onTriggered:
                {
                    PopupUtils.open(readingOptionsPopoverComponent, readingOptionsButton)
                }
            }
        }

        ToolbarButton {
            action: Action {
                text:  rssItem == null ? "" : (rssItem.favourite == "0" ? i18n.tr("Save") : i18n.tr("Remove"))
                iconSource: {
                    if (rssItem == null || rssItem.favourite == "0")
                        return Qt.resolvedUrl("./icons_tmp/favorite-unselected.svg")
                    else return Qt.resolvedUrl("./icons_tmp/favorite-selected.svg")
                }
                onTriggered: {
                    var fav = (rssItem.favourite == "0" ? "1" : "0")
                    var dbResult = DB.updateArticleFavourite(rssItem.id, fav)
                    if (dbResult.rowsAffected == 1) {
                        //rssItem.favourite = fav
                        //var dbResult1 = DB.updateArticleStatus(rssItem.id, "0")
                        //if (dbResult1.rowsAffected == 1) {
                            // comment bellow to disable status signal, all articles will be set as read
                            //articleStatusChanged(rssItem.tagId, rssItem.id, "0" /*rssItem.status*/ )
                        //}

                        articleFavouriteChanged(rssItem, fav)
                    }
                }
            }
        }

        ToolbarButton {
            action: Action {
                text:  i18n.tr("Open site")
                iconSource: Qt.resolvedUrl("./icons_tmp/go-to.svg")
                onTriggered:
                {
                    console.log("Open site", rssListview.model.get(rssListview.currentIndex).link)
                    Qt.openUrlExternally(rssListview.model.get(rssListview.currentIndex).link)
                }
            }
        }
    }

    Component {
        id: readingOptionsPopoverComponent

        Popover {
            id: readingOptionsPopover

            Component.onCompleted: {
//                var useDarkTheme = DB.getUseDarkTheme()
//                console.log("USE DARK", useDarkTheme)
                fontSizeSlider.value = optionsKeeper.fontSize()
                buttonRow.updateButtonsState()
            }

            Column {
                id: contentColumn

                anchors {
                    left: parent.left
                    top: parent.top
                    right: parent.right
                }

                ListItem.Empty {
                    Row {
                        id: buttonRow

                        property bool useDark: false
                        spacing: units.gu(2)
                        anchors.centerIn: parent
                        Button {
                            text: "Dark"
                            width: units.gu(14)
                            gradient: buttonRow.useDark ? UbuntuColors.orangeGradient : UbuntuColors.greyGradient
                            onClicked: {
                                optionsKeeper.setUseDarkTheme(true)
                                buttonRow.updateButtonsState()
                            }
                        }

                        Button {
                            text: "Light"
                            width: units.gu(14)
                            gradient: buttonRow.useDark ? UbuntuColors.greyGradient : UbuntuColors.orangeGradient
                            onClicked: {
                                optionsKeeper.setUseDarkTheme(false)
                                buttonRow.updateButtonsState()
                            }
                        }

                        function updateButtonsState() {
                            useDark = optionsKeeper.useDarkTheme()
                        }
                    }
                }

                ListItem.Empty {
                    showDivider: false
                    Label {
                        id: lblLess

                        text: "A"
                        fontSize: "small"
                        color: "black" // TODO TEMP
                        anchors {
                            left: parent.left
                            leftMargin: units.gu(2)
                            verticalCenter: parent.verticalCenter
                        }
                    }

                    Slider {
                        id: fontSizeSlider

                        anchors {
                            right: lblMore.left
                            rightMargin: units.gu(1)
                            left: lblLess.right
                            leftMargin: units.gu(1)
                            verticalCenter: parent.verticalCenter
                        }

                        minimumValue: 0
                        maximumValue: 2

                        onValueChanged: {
                            var res
                            if (value < maximumValue / 3)
                                res = 0
                            else if (value < maximumValue / 3 * 2)
                                res = 1
                            else res = 2

                            optionsKeeper.setFontSize(res)
                        }

                        function formatValue(v) {
                            if (v < maximumValue / 3)
                                return i18n.tr("Small")
                            else if (v < maximumValue / 3 * 2)
                                return i18n.tr("Mid")
                            else return i18n.tr("Large")
                        }
                    } // SLider

                    Label {
                        id: lblMore

                        text: "A"
                        fontSize: "large"
                        color: "black" // TODO TEMP
                        anchors {
                            right: parent.right
                            rightMargin: units.gu(2)
                            verticalCenter: parent.verticalCenter
                        }
                    }
                } // ListItem.Empty
            }
        }
    } // Component

    //////////////////////////////////////////////      a listview to show the RSS content
    ListView {
        id: rssListview

        width: parent.width
        anchors {
            bottom: parent.bottom
            bottomMargin: units.gu(2)
            top: parent.top
            topMargin: units.gu(2)
        }

        snapMode: ListView.SnapOneItem
        cacheBuffer: 90 // value in pixels, for what?
        // boundsBehavior: Flickable.StopAtBounds
        orientation: ListView.Horizontal
        contentHeight: parent.width * count
        model: rssModel
        delegate: xmlDelegate
        highlightFollowsCurrentItem: true
        highlightRangeMode: ListView.StrictlyEnforceRange

        onCurrentIndexChanged: {
            console.log("ListView onCurrentIndexChanged", currentIndex, preventIndexChangeHandler)
            if (preventIndexChangeHandler) {
                preventIndexChangeHandler = false
                return
            }

            if (rssModel == null || rssModel.get == undefined) {
                console.log("---- Stange behavior ----")
                console.trace()
                return
            }

            if (rssModel.count == 0) // It is normal bevaviour.
                return

            rssItem = rssModel.get(currentIndex)
            rssTitle = rssItem.feed_name

            if (rssItem.status != "1") {
                var dbResult = DB.updateArticleStatus(rssItem.id, "1")
                if (dbResult.rowsAffected == 1) {
                    articleStatusChanged(rssItem.tagId, rssItem.id, "1")
                }
            }
        }
    }

    //////////////////////////////////////////////      delegate for ListView
    Component {
        id: xmlDelegate

        Flickable {
            id: scrollArea

            clip: true

            width: rssListview.width
            height: rssListview.height

            contentWidth: width
            contentHeight: innerAreaColumn.height

            Column {
                id: innerAreaColumn

                spacing: units.gu(2)
                width: parent.width

                property int mediaDownloadInProgress: 0

                function mediaDownloaded() {
                    mediaDownloadInProgress = mediaDownloadInProgress - 1
                }

                Row {
                    width: parent.width - units.gu(4)
                    height: labelTime.paintedHeight
                    anchors.horizontalCenter: parent.horizontalCenter
                    spacing: units.gu(1)

                    Image {
                        id: imgFavourite
                        anchors.verticalCenter: labelTime.verticalCenter
                        fillMode: Image.PreserveAspectCrop
                        source: Qt.resolvedUrl("./icons_tmp/favorite-selected.svg")
                        sourceSize.height: rssItem == null ? 0 : (rssItem.favourite == "1" ? units.gu(2) : 0)
                        visible: rssItem == null ? false : (rssItem.favourite == "1")
                        smooth: true
                    }

                    Label {
                        id: labelTime
                        text: DateUtils.formatRelativeTime(i18n, pubdate)
                        fontSize: "small"
                        width: parent.width - units.gu(3)
                        wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                    }
                }

                Label {
                    fontSize: "large"
                    text: title
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                }

                Label {
                    text: content
                    linkColor: "white"
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                    clip: true

                    onLinkActivated: Qt.openUrlExternally(link)
                }

                Label {
                    id: label_feedname
                    text: rssTitle
                    fontSize: "small"
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                }

                Button {
                    id: showMediaButton

                    text: i18n.tr("Show media")
                    anchors.horizontalCenter: parent.horizontalCenter
                    // Everyone can test my "Show media". I want it in v 1.0 too.
                    visible: false //rssModel.get(index).media_groups ? true : false

                    onClicked: {
                        visible = false

                        var medias = JSON.parse(rssModel.get(index).media_groups)
                        var len = medias.length

                        for (var i = 0; i < len; i++) {
                            var media = medias[i]

                            for (var j = 0; j < media.contents.length; j++) {
                                var cont = media.contents[j]

                                if (cont.type === "image/jpeg" || cont.type === "image/png" ||
                                        cont.type === "image/jpeg" || cont.type === "image/pjpeg" ||
                                        cont.type === "image/svg+xml" || cont.medium === "image") {
                                    var mediaImg = mediaGroupImageComponent.createObject(innerAreaColumn)

                                    innerAreaColumn.mediaDownloadInProgress++
                                    mediaImg.mediaLoadFinished.connect(innerAreaColumn.mediaDownloaded)

                                    mediaImg.anchors.horizontalCenter = showMediaButton.horizontalCenter
                                    mediaImg.source = cont.url
                                }
                            }
                        }
                    }
                }

                ActivityIndicator {
                    id: mediaDownloadIndicator

                    running: innerAreaColumn.mediaDownloadInProgress > 0
                    visible: running // && !showMediaButton.visible && model.articleMediaString != ""
                    anchors.horizontalCenter: parent.horizontalCenter
                }
            }
        } // Flickable
    } // Component

    Component {
        id: mediaGroupImageComponent

        Image {
            id: mediaGroupImage

            signal mediaLoadFinished

            fillMode: Image.PreserveAspectFit
            width: Math.min(sourceSize.width, pageRoot.width - units.gu(4))

            onStatusChanged: {
                if (status == Image.Ready)
                    mediaLoadFinished()
            }
        }
    }
}
