import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1

import "databasemodule_v2.js" as DB
import "listview"

Tab {
    id: topicTab

    property int topicId: -1
    property bool isListMode: pageStack.isListView

    property bool isActive: false

    page: isListMode ? listPage : gridPage

    onIsListModeChanged: {
        if (topicId == -1) // Ignore first undefined state.
            return

        reload()
    }

    function reload() {
        // console.log('TopicTab reload', topicId)
        clear()

        var articlesByTopic = []
        var topicArticles = DB.loadArticles({"tagId" : topicId})

        for (var j = 0; j < topicArticles.rows.length; j++) {
            var ca = topicArticles.rows.item(j)

            var anObj = {"tagName" : "",
                "tagId" : topicId,
                "title" : ca.title,
                "content" : ca.content,
                "link" : ca.link,
                "description" : ca.description,
                "pubdate" : ca.pubdate,
                "status" : ca.status, // ?
                "favourite" : ca.favourite,
                "feedId" : ca.feed_id,
                "image" : ca.image,
                "feed_name" : ca.feed_name,
                "media_groups" : ca.media_groups,
                "id" : ca.id }

            articlesByTopic.push(anObj)
        }

        // Sort for list mode.
        if (isListMode) {
            articlesByTopic.sort(function(a,b) {
                if (a.feedId === b.feedId)
                    return b.pubdate - a.pubdate
                else return b.feedId - a.feedId
            })
        }

        for(var i = 0; i < articlesByTopic.length; i++)
            commonModel.append(articlesByTopic[i])
    }

    function clear() {
        isActive = false
        gridPage.clear()
        listPage.clear()
        commonModel.clear()
    }

    function showContent() {
        console.log("showContent: ", topicId, isActive)
        if (isActive)
            return

        isActive = true
        if (isListMode)
            listPage.reload()
        else gridPage.reload()
    }

    /* All views should use this method to access shared model.
     */
    function getModel() {
        return commonModel
    }

    function updateStatusInModel(articleId, status) {
        for ( var i = 0; i < commonModel.count; i++) {
            if (commonModel.get(i).id == articleId) {
                commonModel.get(i).status = status
                break
            }
        }
    }

    function updateFavouriteInModel(articleId, fav) {
        for ( var i = 0; i < commonModel.count; i++) {
            if (commonModel.get(i).id == articleId) {
                commonModel.get(i).favourite = fav
                return
            }
        }
    }

    ListModePage {
        id: listPage

        visible: isListMode
        topicId: topicTab.topicId

        tools: mainCommonToolbar
    }

    GridModePage {
        id: gridPage

        visible: !isListMode
        topicId: topicTab.topicId

        tools: mainCommonToolbar
    }

    ListModel {
        id: commonModel
    }

    Label {
        id: lblEmptyBase

        text: i18n.tr("There are no articles to show")
        anchors.centerIn: parent
        visible: commonModel.count == 0
        fontSize: "large"
    }
}
