import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import Ubuntu.Layouts 0.1

import "../databasemodule_v2.js" as DB

Page {
    id: editPage
    objectName: "editfeedpage"
    title: i18n.tr("Edit Feed")
    flickable: null/*content*/
    tools: null

    ToolbarItems {
        id: toolbar

        opened: true
        locked: true

        back: Button {
            id: cancelButton
            text: i18n.tr("Cancel")
            gradient: UbuntuColors.greyGradient
            anchors.verticalCenter: parent.verticalCenter
            onClicked: {
                pageStack.pop()
            }
        }

        Button
        {
            id: btnDelete
            objectName: "deleteButton"
            text: i18n.tr("Delete")
            gradient: UbuntuColors.orangeGradient
            anchors.verticalCenter: parent.verticalCenter
            visible: false // disable it from bug #1224892

            onClicked:
            {
                DB.deleteFeed(feedId) ;
                deleteFeed(feedId, previousTopicId) ;
                pageStack.pop() ;
            }
        }

        Button
        {
            id: btnApply
            objectName: "doneButton"
            text: i18n.tr("Done")
            gradient: UbuntuColors.orangeGradient
            anchors.verticalCenter: parent.verticalCenter
            visible: previousTopicId != newTopicId

            onClicked:
            {
                if (previousTopicId != newTopicId) {
                    //                DB.updateFeedByUser(feedId, feedTitle, feedURL) ;
                    DB.deleteFeedTag(feedId, previousTopicId) ;
                    DB.addFeedTag(feedId, newTopicId) ;
                    apply(feedId, newTopicId, previousTopicId) ;
                    pageStack.pop() ;
                }
            }
        }
    }

    signal apply(int feedId, int newTopicId, int previousTopicId)
    signal deleteFeed(int feedId, int topicId)

    property int feedId
    property string feedTitle: ""
    property string feedURL: ""
    property int previousTopicId
    property int newTopicId
    property var dbTags
    property var topicArray

    function setValues(feedid, title, url, pTopicId)
    {
        feedId = feedid ;
        feedTitle = title ;
        feedURL = url ;
        previousTopicId = pTopicId ;
        newTopicId = pTopicId
        topicArray = [] ;
        var tags = DB.loadTags() ;
        var tArray = [] ;
        var tagsArray = [] ;
        var index
        for (var i=0; i<tags.rows.length; i++)
        {
            if(tags.rows[i].id == previousTopicId)
            {
                index = i ;
            }
            tArray.push(tags.rows[i].name) ;
            tagsArray.push(tags.rows[i]) ;
        }
        dbTags = tagsArray ;
        topicArray = tArray ;
        seletorTopic.selectedIndex = index ;
    }

    function reloadPageContent() {
        editPage.tools = toolbar
    }

    Flickable {
        id: content
        anchors { fill: parent; topMargin: units.gu(2)/*margins: units.gu(2)*/ }
        contentHeight: contentItem.childrenRect.height
        boundsBehavior: (contentHeight > editPage.height) ? Flickable.DragAndOvershootBounds : Flickable.StopAtBounds

        Column
        {
            anchors{ left: parent.left; right: parent.right }
            spacing: units.gu(2)

            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Label
                {
                    id: labelTitle
                    text: i18n.tr("Title: ")
                    width: units.gu(6)
                    anchors.verticalCenter: parent.verticalCenter
                }

                TextField
                {
                    text: feedTitle
                    width: parent.width - labelTitle.width - parent.spacing
                    anchors.verticalCenter: parent.verticalCenter
                    readOnly: true
                    hasClearButton: false
                }
            }

            Row
            {
                anchors{ left: parent.left; right: parent.right; leftMargin: units.gu(2); rightMargin: units.gu(2) }
                spacing: units.gu(1)

                Label
                {
                    id: labelURL
                    text: i18n.tr("URL: ")
                    width: labelTitle.width
                    anchors.verticalCenter: parent.verticalCenter
                }

                TextField
                {
                    text: feedURL
                    width: parent.width - labelURL.width - parent.spacing
                    anchors.verticalCenter: parent.verticalCenter
                    readOnly: true
                    hasClearButton: false
                }
            }

            ListItem.ValueSelector {
                objectName: "valueselector"
                id: seletorTopic
                text: i18n.tr("Topic: ")
                values: (topicArray == undefined || topicArray.length == 0) ? [""] : topicArray
//                selectedIndex: 0

                onSelectedIndexChanged:
                {
                    var tArray = topicArray ;
                    var topicname = tArray[seletorTopic.selectedIndex] ;
                    var tags = dbTags ;
                    console.log("detail: ", JSON.stringify(tags)) ;
                    for (var i=0; i<tags.length; i++)
                    {
                        if(tags[i].name == topicname)
                        {
                            newTopicId = tags[i].id ;
                            break ;
                        }
                    }
                    console.log("new topic id: ", newTopicId) ;
                }
            }
        }
    }
}
