# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from ubuntuuitoolkit import emulators as toolkit_emulators
from time import sleep


class MainView(toolkit_emulators.MainView):

    retry_delay = 0.2

    #ideally this should be converted to the sdk
    #and use object name not text names
    #see https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1205144
    def run_action(self, popover, name):
        actions = popover.select_many("Empty")
        requested = None
        for action in actions:
            if action.text == name:
                requested = action
        self.pointing_device.click_object(requested)

    def select_many_retry(self, object_type, **kwargs):
        """Returns the item that is searched for with app.select_many
        In case of no item was not found (not created yet) a second attempt is
        taken 1 second later"""
        items = self.select_many(object_type, **kwargs)
        tries = 10
        while len(items) < 1 and tries > 0:
            sleep(self.retry_delay)
            items = self.select_many(object_type, **kwargs)
            tries = tries - 1
        return items

    def get_action_popover(self):
        # Returns all instances, but with current one as first index
        return self.select_many_retry("ActionSelectionPopover")[0]

    def get_network_activity(self):
        try:
            activity = self.select_single("ActivityIndicator", running="True")
        except:
            return
        return activity

    def strip_html_tags(self, text):
        finished = 0
        while not finished:
            finished = 1
            start = text.find("<")
            if start >= 0:
                stop = text[start:].find(">")
                if stop >= 0:
                    text = text[:start] + text[start+stop+1:]
                    finished = 0
        return text

    #### choose topics page objects
    def get_choose_topics_items(self):
        return self.select_many("Standard", objectName="topicItem")

    def get_new_topic_input_box(self):
        return self.select_single("TextField", objectName="newTopic")

    #### append feed page objects
    def get_append_feed_input_box(self):
        return self.select_single("TextField", objectName="tfFeedUrl")

    def get_append_page_feed(self):
        #grab just the first checkbox
        return self.select_many_retry("CheckBox", objectName="feedCheckbox")[0]

    def get_append_page_list_items(self):
        return self.select_many_retry("Standard",
                                      objectName="feedpagelistitem")

    def get_next_button(self):
        return self.select_single("Button", objectName="nextButton")

    #### feed list page objects
    def get_feedlist_topic(self, name):
        return self.wait_select_single("TopicComponent", topicName=name)

    def get_feedlist_feed(self, feed):
        return self.wait_select_single("FeedComponent", text=feed)

    def get_add_a_topic(self):
        return self.select_single("QQuickItem", objectName="addTopic")

    #### edit feed page objects
    def get_delete_button(self):
        return self.select_single("Button", objectName="deleteButton")

    #### Page Calls
    def get_append_page(self):
        return self.select_single("AppendFeedPage",
                                  objectName="appendfeedpage")

    def get_topics_page(self):
        return self.select_single("ChooseTopicPage",
                                  objectName="choosetopicpage")

    def get_feedlist_page(self):
        return self.wait_select_single("FeedListPage",
                                       objectName="feedlistpage")

    def get_articlelist_page(self):
        return self.select_single("ArticleListPage",
                                  objectName="articlelistpage")

    def get_feed_management_page(self):
        return self.wait_select_single("TopicManagement",
                                       objectName="topicmanagement")

    def get_edit_feed_page(self):
        return self.select_single("EditFeed",
                                  objectName="editfeedpage")

    def get_manage_topics_page(self):
        return self.select_single("ManageTopicsPage")

    def get_rss_feed_page(self):
        return self.wait_select_single(
            "RssFeedPage", objectName="rssfeedpage")

    def get_shorts_tab(self):
        return self.wait_select_single(
            "ShortsTab", objectName="Tab0")

    def get_topics(self):
        return self.select_single("QQuickListView", objectName="topiclist")

    def get_feeds(self):
        return self.select_many("Standard", objectName="FeedListItem")

    def get_header(self):
        return self.select_many("Header")

    def get_all_feeds(self):
        return self.select_single("Subtitled", text="All articles")

    def get_refresh_button(self):
        return self.select_single("AbstractButton", objectName="refreshbutton")

    def get_list_view_button(self):
        return self.select_single("AbstractButton",
                                  objectName="listviewbutton")

    def get_add_feed_button(self):
        return self.select_single("AbstractButton", objectName="addfeedbutton")

    def get_edit_topic_button(self):
        return self.select_single("AbstractButton",
                                  objectName="edittopicbutton")

    def get_refresh_feed_dialog(self):
        return self.select_single("Dialog", objectName="refreshWaitDialog")

    def get_refresh_dialog_cancel_button(self):
        return self.select_single("Button", objectName="refreshCancel")

    def get_toolbar_back_button(self):
        return self.select_single("ActionItem", text="Back")

    def get_editfeed_done_button(self):
        return self.wait_select_single("Button", objectName="doneButton")

    def get_topic_tab(self, topic):
        return self.wait_select_single("TopicTab", title=topic)

    def get_feed_in_feedlist(self, topic, feed):
        itemList = self.get_feedlist(topic)
        for item in itemList:
            label = item.select_single("Label", objectName="labelFeedname")
            if label.text == feed:
                return item

    def get_feedlist(self, topic):
        #tab = self.get_topic_tab(topic)
        items = self.select_many("UbuntuShape", objectName="feedlistitems")
        return items

    def get_editfeed_topic_valueselector(self):
        return self.wait_select_single(
            "ValueSelector", objectName="valueselector")

    def get_editfeed_valueselector_value(self, name):
        valueselector = self.get_editfeed_topic_valueselector()
        return valueselector.select_single("LabelVisual", text=name)
