# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""shorts app autopilot tests."""

from __future__ import absolute_import

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, Is, Not

from shorts_app.tests import ShortsAppTestCase
from time import sleep
import unittest
import logging

logger = logging.getLogger(__name__)


class TestMainWindow(ShortsAppTestCase):

    def setUp(self):
        super(TestMainWindow, self).setUp()

        self.assertThat(self.main_view.visible, Eventually(Equals(True)))
        #wait for any updates to finish before beginning tests
        self._wait_for_refresh()

    def _wait_for_refresh(self):
        try:
            self.assertThat(
                self.main_view.get_network_activity,
                Eventually(NotEquals(None), timeout=60))
            self.assertThat(
                self.main_view.get_network_activity,
                Eventually(Equals(None), timeout=60))
        except:
            self.assertThat(
                self.main_view.get_network_activity,
                Eventually(Equals(None), timeout=60))

    def _add_feed_and_topic(self, feed, topicName):
        """test add feed and topic"""
        logger.debug("starting test add feed and topic")

        self.assertThat(self.main_view.get_topics_page,
                        Eventually(NotEquals(None)))
        #open toolbar
        logger.debug("opening toolbar")
        toolbar = self.main_view.open_toolbar()

        #click on add reads toolbar button
        logger.debug("clicking addreadsbutton")
        toolbar.click_button("addreadsbutton")

        #click on add feeds popover action
        logger.debug("clicking + Add feeds")
        addReadsPopover = self.main_view.get_action_popover()
        self.main_view.run_action(addReadsPopover, "+ Add feeds")

        #verify Append Feed  page is open
        logger.debug("asserting append page")
        appendpage = lambda: self.main_view.get_append_page().visible
        self.assertThat(appendpage, Eventually(Equals(True)))

        #type feed in input field
        logger.debug("inputting feed")
        self._input_new_feed(feed)

        #click on checkbox to confirm feed
        logger.debug("asserting checkbox")
        self.assertThat(self.main_view.get_append_page_feed,
                        Eventually(Not(Is(None))))
        logger.debug("grabbing checkbox")
        checkboxBtn = self.main_view.get_append_page_feed()
        logger.debug("clicking checkbox")
        self.pointing_device.click_object(checkboxBtn)
        logger.debug("asserting checkbox checked")
        self.assertThat(checkboxBtn.checked, Eventually(Equals(True)))

        #save selected feedname
        feedName = self.main_view.strip_html_tags(
            str(self.main_view.get_append_page_list_items()[0].text))

        #click on Next button
        logger.debug("asserting next")
        self.assertThat(self.main_view.get_next_button,
                        Eventually(Not(Is(None))))
        logger.debug("grabbing next")
        nextButton = self.main_view.get_next_button()
        logger.debug("clicking next")
        self.pointing_device.click_object(nextButton)

        #verify choose topic page has opened
        logger.debug("asserting choose topic page")
        chossetopicpage = lambda: self.main_view.get_topics_page().visible
        self.assertThat(chossetopicpage, Eventually(Equals(True)))

        #get current topics
        logger.debug("grabbing topics")
        topics = self.main_view.get_choose_topics_items()
        numTopics = len(topics)
        logger.debug("found " + str(numTopics) + " topics")
        for topic in topics:
            logger.debug("Topic: " + str(topic.text))

        #add a new topic
        logger.debug("input topic")
        self._input_new_topic(topicName)

        #wait for new feed and topic to be added
        self._wait_for_refresh()

        return feedName

    def _remove_feed(self, feedName, topicName):
        logger.debug("removing feed")
        #open toolbar
        logger.debug("opening toolbar")
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar Button
        logger.debug("clicking edit topics")
        toolbar.click_button("edittopicsbutton")

        #verify we are on the Management page
        logger.debug("assert feedlist page")
        feedlistpage = self.main_view.get_feed_management_page()
        self.assertThat(feedlistpage.visible, Eventually(Equals(True)))

        ##select feed
        logger.debug("assert feed")
        self.assertThat(self.main_view.get_feedlist_feed(feedName).text,
                        Equals(feedName))
        logger.debug("grabbing feed")
        feed = self.main_view.get_feedlist_feed(feedName)
        feedLabel = feed.select_single('Label', text=feedName)

        self.assertThat(self.main_view.get_feed_management_page,
                        Eventually(NotEquals(None)))

        #swipe to delete feed
        startX = int(feedlistpage.x + feedlistpage.width * 0.30)
        stopX = int(feedlistpage.x + feedlistpage.width)
        lineY = int(feedLabel.globalRect[1])

        #swipe to remove the feed
        self.pointing_device.move(startX, lineY)
        self.pointing_device.drag(startX, lineY, stopX, lineY)

        #verify we are on the Management page
        logger.debug("assert feedlistpage")
        feedlistpage = lambda: self.main_view. \
            get_feed_management_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #check topic
        logger.debug("assert topic remains")
        # If the topic is not present, it will raise an error.
        self.main_view.get_feedlist_topic(topicName)

        #check feed has been removed
        logger.debug("assert removed feed")
        self.assertThat(
            lambda: self.main_view.select_many('FeedComponent', text=feedName),
            Eventually(Equals([])))

    def _remove_topic(self, topicName):
        self.assertThat(self.main_view.get_topics_page,
                        Eventually(NotEquals(None)))

        #open toolbar
        toolbar = self.main_view.open_toolbar()
        self.assertThat(self.main_view.open_toolbar,
                        Eventually(NotEquals(None)))

        #click on back toolbar Button
        backButton = self.main_view.get_toolbar_back_button()
        self.pointing_device.click_object(backButton)
        self.assertThat(self.main_view.get_topics_page,
                        Eventually(NotEquals(None)))

        #open toolbar
        toolbar = self.main_view.open_toolbar()
        self.assertThat(self.main_view.open_toolbar,
                        Eventually(NotEquals(None)))

        #click on edit topics button
        toolbar.click_button("edittopicsbutton")

        #verify we are on the Management page
        feedlistpage = self.main_view.get_feed_management_page()
        self.assertThat(feedlistpage.visible, Eventually(Equals(True)))

        #select topic
        topic = self._get_topic(topicName)
        topicLabel = topic.select_single("Label", text=topicName)

        topicmanagementpage = self.main_view.get_feed_management_page()
        self.assertThat(self.main_view.get_feed_management_page,
                        Eventually(NotEquals(None)))

        #swipe to delete topic
        startX = int(topicmanagementpage.x + topicmanagementpage.width * 0.30)
        stopX = int(topicmanagementpage.x + topicmanagementpage.width)
        lineY = topicLabel.globalRect[1]

        #swipe to remove the topic
        self.pointing_device.move(startX, lineY)
        self.pointing_device.click()
        self.pointing_device.drag(startX, lineY, stopX, lineY)

        #verify we are on the Topic Management page **for timing issue
        topicmanagementpage = self.main_view.get_feed_management_page()
        self.assertThat(self.main_view.get_feed_management_page,
                        Eventually(NotEquals(None)))
        self.assertThat(self.main_view.get_feed_management_page().visible,
                        Eventually(Equals(True)))
        self.assertThat(self.main_view.get_feed_management_page().objectName,
                        Eventually(Equals("topicmanagement")))

        #check topic has been removed
        logger.debug("assert removed topic")
        try:
            self.assertThat(self._get_topic(topicName),
                            Eventually(Equals(None)))
        except:
            logger.debug("Removed topic %s" % topicName)

    def _input_new_feed(self, rssFeed):
        inputField = self.main_view.get_append_feed_input_box()
        return self._safe_enter_text(inputField, rssFeed)

    def _input_new_topic(self, topic):
        inputField = self.main_view.get_new_topic_input_box()
        return self._safe_enter_text(inputField, topic)

    def _safe_enter_text(self, inputField, text, timeout=10):
        #poll clicking for focus for lack of better method,
        #due to screen switching animation
        #is there a property we can use instead?
        poll = 0
        while not inputField.focus and poll < timeout:
            self.pointing_device.click_object(inputField)
            sleep(1)
            poll += 1
        self.keyboard.type(text)
        self.assertThat(inputField.text, Eventually(Equals(text)))
        self.keyboard.press_and_release("Enter")
        return inputField

    def _get_topic(self, topicName, timeout=10):
        logger.debug("_get_topic: assert topics")
        self.assertThat(lambda: self.main_view.get_feedlist_topic(topicName),
                        Eventually(Not(Is([]))))
        logger.debug("_get_topic: grab topic")
        topic = self.main_view.get_feedlist_topic(topicName)
        logger.debug("_get_topic: click topic")

        poll = 0
        while not topic.isExpended and poll < timeout:
            logger.debug("_get_topic: loop click topic")
            self.pointing_device.click_object(topic)
            sleep(1)
            poll += 1
        return topic

    def test_add_remove_feed_and_topic(self):
        """ test add and remove feed and topic"""

        #add topic and feed
        #TODO, try several different feeds
        feed = "http://www.canonical.com/rss.xml"
        feedName = "Insights"
        topicName = "CanonicalTopic"

        #just for timing issues
        self.assertThat(self.main_view.get_shorts_tab(),
                        NotEquals(None))
        self.assertThat(self.main_view.get_shorts_tab().objectName,
                        Eventually(Equals("Tab0")))
        self.assertThat(self.main_view.get_shorts_tab().visible,
                        Eventually(Equals(True)))

        #add feed and topic
        feedName = self._add_feed_and_topic(feed, topicName)

        #remove added feed
        logger.debug("starting remove feed")

        self._remove_feed(feedName, topicName)

        #remove added topic
        self._remove_topic(topicName)

    @unittest.skip("Tab switching fails")
    def test_view_mode_and_feed_item(self):
        """test view modes and ensure feed items can be opened"""

        feed = "http://www.canonical.com/rss.xml"
        #feedName = "Insights"
        topicName = "CanonicalTopic"

        #just for timing issues
        self.assertThat(self.main_view.get_shorts_tab(),
                        NotEquals(None))
        self.assertThat(self.main_view.get_shorts_tab().objectName,
                        Eventually(Equals("Tab0")))
        self.assertThat(self.main_view.get_shorts_tab().visible,
                        Eventually(Equals(True)))

        #add topic and feed
        self._add_feed_and_topic(feed, topicName)

        #verify we are on the editTopic Tab
        editTab = self.main_view.get_topic_tab(topicName)
        self.assertThat(lambda: editTab.visible, Eventually(Equals(True)))

        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on refresh  toolbar button
        toolbar.click_button("rsspagerefreshbutton")

        #wait for any updates to finish before continuing tests
        self._wait_for_refresh()

        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on list view toolbar button
        toolbar.click_button("changemodebutton")

        #TODO FIX WHY THIS WON'T SWITCH TO THE PROPER TAB
        #switch to the feed
        #header = self.main_view.get_header()
        #header.switch_to_next_tab()
        #self.main_view.get_header().switch_to_next_tab()
        #feedTab = self.main_view.get_topic_tab(topicName)
        #self.pointing_device.click_object(feedTab)
        #self.assertThat(lambda: feedTab.visible, Eventually(Equals(True)))

        #select feed entry
        self.assertThat(lambda: self.main_view.get_feedlist(topicName),
                        Eventually(Not(Is(None))))
        editFeed = self.main_view.get_feedlist(topicName)

        for item in editFeed:
            label = item.select_single("Label", objectName="labelFeedname")
            logger.debug("editFeed: " + str(label.text))

        #click on the first entry in the list
        self.pointing_device.click_object(editFeed[0])

        #verify we are on RSS feed page
        rssfeedpage = lambda: self.main_view.get_rss_feed_page().visible
        self.assertThat(rssfeedpage, Eventually(Equals(True)))

    @unittest.skip("Can't see or get dynamic tabs")
    def test_edit_topic(self):
        """test edit topic"""

        feed = "http://www.canonical.com/rss.xml"
        feedName = "Insights"
        topicName = "CanonicalTopic"

        #add topic and feed
        feedName = self._add_feed_and_topic(feed, topicName)

        #verify we are on the topicName Tab
        editTab = self.main_view.get_topic_tab(topicName)
        self.assertThat(editTab.visible, Eventually(Equals(True)))

        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on edit topics button
        toolbar.click_button("edittopicsbutton")

        # verify we are on topic management page
        topicmanagementpage = self.main_view.get_feed_management_page()
        self.assertThat(
            topicmanagementpage.objectName,
            Eventually(Equals("topicmanagement")))

        #verity Tab is expanded
        editTopic = self.main_view.get_feedlist_topic(topicName)
        if editTopic.state != "expended":
            self.pointing_device.click_object(editTopic)

        #select feed
        editFeed = self.main_view.get_feedlist_feed(feedName)
        self.assertThat(editFeed, NotEquals(None))
        self.pointing_device.click_object(editFeed)

        #verify we are on edit feed page
        editfeedpage = self.main_view.get_edit_feed_page()
        self.assertThat(editfeedpage.visible, Eventually(Equals(True)))

        #change topic from dropdown
        topicValueselector = self.main_view.get_editfeed_topic_valueselector()
        self.pointing_device.click_object(topicValueselector)

        #click on news topic
        topicValueselectorValue = self.main_view. \
            get_editfeed_valueselector_value("Ubuntu")
        self.pointing_device.click_object(topicValueselectorValue)

        #verify topic has changed
        self.assertThat(topicValueselectorValue.text,
                        Eventually(Equals("Ubuntu")))
